<?php
namespace Ak\Test;

use Ak\Test\Constraint\Container\ServiceExists;
use Ak\Test\Constraint\Container\ServiceIsInstanceOf;
use Ak\Test\Constraint\Container\ServiceIsNotShared;
use Ak\Test\Constraint\Container\ServiceIsShared;
use Laminas\ConfigAggregator\ArrayProvider;
use Laminas\ConfigAggregator\ConfigAggregator;
use Laminas\ServiceManager\ServiceManager;
use Laminas\Stdlib\ArrayUtils;
use PHPUnit\Framework\TestCase;
use Psr\Container\ContainerInterface;

/**
 * Osnovna klasa za testove.
 *
 * @package Ak\Test
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/test
 */
abstract class AbstractTestCase extends TestCase
{
    /**
     * ConfigProvideri komponenti koje ovaj test koristi.
     */
    protected array $providers = [];

    /**
     * Zadnji generirani container.
     */
    private ?ServiceManager $container = null;

    /**
     * Postavlja test
     */
    protected function setUp(): void
    {
        parent::setUp();
        $this->container = null;
    }

    //<editor-fold desc="1. Konfiguracija">
    /**
     * Vraća konfiguraciju sakupljenu iz sljedećih izvora:
     *
     * 1. ConfigProvideri postavljeni u $this->configProviders.
     * 2. Konfiguraciju poslanu kao argument metode.
     *
     * @param array $config
     *
     * @return array
     */
    public function getConfig(array $config = []): array
    {
        return (new ConfigAggregator(ArrayUtils::merge($this->providers, [
            new ArrayProvider($config),
        ])))->getMergedConfig();
    }
    //</editor-fold>
    //<editor-fold desc="2. Container">
    /**
     * Vraća konfigurirani container. Svaki poziv vraća novu instancu.
     *
     * @param array $config
     *
     * @param array $services
     * @return ServiceManager
     */
    public function getContainer(array $config = [], array $services = []): ServiceManager
    {
        if (null === $this->container) {
            $config = $this->getConfig($config);

            if (isset($config['dependencies']['services']) && is_array($config['dependencies']['services'])) {
                $config['dependencies']['services'] = array_merge($config['dependencies']['services'], $services);
            } else {
                $config['dependencies']['services'] = $services;
            }

            $this->container = new ServiceManager($config['dependencies'] ?? []);
            $this->container->setService('config', $config);
        }

        return $this->container;
    }

    /**
     * Utvrđuje da dati container sadrži servis s datim imenom.
     *
     * @param ContainerInterface $container
     * @param string|array $nameOrNames
     * @param string|null $instanceOf
     * @param bool $shouldBeShared
     * @param string $message
     */
    public static function assertContainerHasService(
        ContainerInterface $container,
        $nameOrNames,
        string $instanceOf = null,
        bool $shouldBeShared = true,
        string $message = ''
    ) {
        $constraint = new ServiceExists($nameOrNames);
        static::assertThat($container, $constraint, $message);

        if (null !== $instanceOf) {
            $constraint = new ServiceIsInstanceOf($nameOrNames, $instanceOf);
            static::assertThat($container, $constraint, $message);
        }

        $constraint = $shouldBeShared
            ? new ServiceIsShared($nameOrNames)
            : new ServiceIsNotShared($nameOrNames);

        static::assertThat($container, $constraint, $message);
    }
    //</editor-fold>
}
