<?php
namespace AkTest\Test;

use Composer\Autoload\ClassLoader;
use Laminas\ServiceManager\Factory\InvokableFactory;
use PHPUnit\Framework\AssertionFailedError;
use Psr\Container\ContainerInterface;
use stdClass;

/**
 * Class AbstractTestCaseTest
 *
 * @package AkTest\Test
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/test
 */
class ContainerTest extends TestCase
{
    public function testCanCreateContainer()
    {
        $test = $this->getTestUnderTest([
            TestConfigProvider::class,
        ]);

        $container = $test->getContainer();

        $this->assertInstanceOf(ContainerInterface::class, $container);
        $this->assertTrue($container->has('config'));
        $this->assertIsArray($container->get('config'));
    }

    public function testCanInjectServices()
    {
        $test = $this->getTestUnderTest();
        $service = new stdClass();

        $container = $test->getContainer([
            'dependencies' => [
                'services' => [
                    'some_service' => $service,
                ],
            ],
        ]);

        $this->assertTrue($container->has('some_service'));
        $this->assertSame($service, $container->get('some_service'));
    }

    public function testContainerHasServiceWillFail()
    {
        $test = $this->getTestUnderTest();

        $container = $test->getContainer();

        $this->expectException(AssertionFailedError::class);
        $test->assertContainerHasService($container, 'invalid');
    }

    public function testContainerHasServiceWorks()
    {
        $test = $this->getTestUnderTest();
        $service = new stdClass();

        $container = $test->getContainer([
            'dependencies' => [
                'services' => [
                    'service' => $service,
                ],
            ],
        ]);

        $test->assertContainerHasService($container, 'service');
    }

    public function testContainerHasServiceWillFailWhenOneOfTheNamesDoesntExist()
    {
        $test = $this->getTestUnderTest();
        $service = new stdClass();

        $container = $test->getContainer([
            'dependencies' => [
                'aliases' => [
                    'alias_1' => 'service',
                ],

                'services' => [
                    'service' => $service,
                ],
            ],
        ]);

        $this->expectException(AssertionFailedError::class);
        $test->assertContainerHasService($container, [
            'alias_1',
            'alias_2',
            'service',
        ]);
    }

    public function testContainerHasServiceWorksWithAliases()
    {
        $test = $this->getTestUnderTest();
        $service = new stdClass();

        $container = $test->getContainer([
            'dependencies' => [
                'aliases' => [
                    'alias_1' => 'service',
                ],

                'services' => [
                    'service' => $service,
                ],
            ],
        ]);

        $test->assertContainerHasService($container, [
            'alias_1',
            'service',
        ]);
    }

    public function testContainerHasServiceWillFailForInstanceCheck()
    {
        $test = $this->getTestUnderTest();
        $service = new stdClass();

        $container = $test->getContainer([
            'dependencies' => [
                'services' => [
                    'service' => $service,
                ],
            ],
        ]);

        $this->expectException(AssertionFailedError::class);
        $test->assertContainerHasService($container, 'service', ClassLoader::class);
    }

    public function testContainerHasServiceWillCheckInstanceOf()
    {
        $test = $this->getTestUnderTest();
        $service = $this->createMock(ClassLoader::class);

        $container = $test->getContainer([
            'dependencies' => [
                'services' => [
                    'service' => $service,
                ],
            ],
        ]);

        $test->assertContainerHasService($container, 'service', ClassLoader::class);
    }

    public function testContainerHasServiceWillFailWhenServiceNotShared()
    {
        $test = $this->getTestUnderTest();
        $service = $this->createMock(ClassLoader::class);
        $service2 = $this->createMock(ClassLoader::class);

        $container = $test->getContainer([
            'dependencies' => [
                'aliases' => [
                    'alias_1' => 'service',
                    'alias_2' => 'service2',
                ],

                'services' => [
                    'service' => $service,
                    'service2' => $service2,
                ],
            ],
        ]);

        $this->expectException(AssertionFailedError::class);
        $test->assertContainerHasService($container, ['service', 'alias_1', 'alias_2'], ClassLoader::class, true);
    }

    public function testContainerHasServiceCheckForSharedService()
    {
        $test = $this->getTestUnderTest();
        $service = $this->createMock(ClassLoader::class);

        $container = $test->getContainer([
            'dependencies' => [
                'aliases' => [
                    'alias_1' => 'service',
                    'alias_2' => 'service',
                ],

                'services' => [
                    'service' => $service,
                ],
            ],
        ]);

        $test->assertContainerHasService($container, ['service', 'alias_1', 'alias_2'], ClassLoader::class, true);
    }

    public function testWillCheckForNonSharedServices()
    {
        $test = $this->getTestUnderTest();
        $container = $test->getContainer([
            'dependencies' => [
                'aliases' => [
                    'alias_1' => stdClass::class,
                    'alias_2' => stdClass::class,
                ],

                'factories' => [
                    stdClass::class => InvokableFactory::class,
                ],
            ],
        ]);

        $this->expectException(AssertionFailedError::class);
        $test->assertContainerHasService($container, ['alias_1', 'alias_2', stdClass::class], stdClass::class, false);
    }

    public function testWillUseTheSameContainerForOneTest()
    {
        $test = $this->getTestUnderTest();

        $container = $test->getContainer();
        $this->assertSame($container, $test->getContainer());

        $test->setUp();

        $this->assertNotSame($container, $test->getContainer());
    }
}
